Package: dcTensor
Type: Package
Title: Discrete Matrix/Tensor Decomposition
Version: 1.3.1
Date: 2025-08-25
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                      email = "k.t.the-answer@hotmail.co.jp"))
Depends: R (>= 3.4.0)
Imports: methods, MASS, fields, rTensor, nnTensor
Suggests: knitr, rmarkdown, testthat
Description: Semi-Binary and Semi-Ternary Matrix Decomposition are performed based on Non-negative Matrix Factorization (NMF) and Singular Value Decomposition (SVD). For the details of the methods, see the reference section of GitHub README.md <https://github.com/rikenbit/dcTensor>.
License: MIT + file LICENSE
URL: https://github.com/rikenbit/dcTensor
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-08-25 01:46:45 UTC; koki
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2025-08-25 02:50:09 UTC
Built: R 4.6.0; ; 2026-04-26 12:39:30 UTC; unix
