importFrom(stats, runif, rpois)
importFrom(graphics, layout, plot.new)
importFrom(grDevices, dev.off, png)
importFrom(methods, is)
importFrom(MASS, ginv)
importFrom(fields, image.plot)
importFrom(rTensor, as.tensor, vec, cs_unfold, khatri_rao, rand_tensor, kronecker_list, fnorm, ttm)
importFrom(nnTensor, NMF, recTensor, plotTensor3D)

export(
    toyModel,
    dNMF,
    dNMTF,
    dSVD,
    dsiNMF,
    djNMF,
    dPLS,
    dNTF,
    dNTD
    )